program Newton_Cotttes
  implicit none
  real, dimension(:), allocatable  :: yn
  real :: a,b, t, h, f1, f2, f3, Mf, Tf, Sf, Ef, f4
  real :: M_E1, T_E1, S_E1
  real :: M_E2, T_E2, S_E2
  integer :: n, loop, nloops, i,j,l, case
  
  
  ! interval
  a = 0.
  b = 1.

  ! number of repeated splittings
  nloops = 7
  
  ! initial number of subintervals
  n = 5

  do loop = 1, nloops
  
     ! size of an subinterval
     h = (b-a) / n
     
     ! end points of subintervals
     allocate(yn(0:n) )
     do i=0,n
        yn(i) = a + i*h
     enddo
     
     
     ! computing over subintervals
     Mf = 0.
     Tf = 0.
     Sf = 0.
     Ef = 0.
     
     do i=1, n
        call Midpoint(yn(i-1), yn(i), f1)
        Mf = Mf + f1
        
        call Trapezoid(yn(i-1), yn(i), f2)
        Tf = Tf + f2
        
        call Simpson(yn(i-1), yn(i), f3)
        Sf = Sf + f3
        
        call Exact(yn(i-1), yn(i), f4)
        Ef = Ef + f4
        
        !write(*,'(i5, 30es12.4)') i, Mf, TF, Sf, Ef, F1, f2, f3, f4
     enddo

     ! errors of computations
     M_E1 = abs( Ef- Mf)
     T_E1 = abs( Ef- Tf)
     S_E1 = abs( Ef- Sf)
     
     if(loop == 1) then
        write(*,'( a6, 4a12, 6a10)') 'h', 'exact',  'M(f)', 'T(f)', 'S(f)', &
             ' errM(f)', ' errT(f)', ' errS(f)', 'estim'
        write(*,*)'-----------------------------------------------------------------',&
             '---------------------------------'
     endif
       
     write(*, 999) h, Ef, Mf, TF, Sf, M_E1, T_E1, S_E1, abs(Mf - Tf)/3
     if(loop > 1) then
     !   write(*,998) &
     !        log(M_E2/M_E1) / log(2.), log(T_E2/T_E1) / log(2.), log(S_E2/S_E1) / log(2.)
     !   write(*,'(x)')
     endif
     ! storing of the old values
     M_E2 = M_E1
     T_E2 = T_E1
     S_E2 = S_E1
     
     deallocate (yn)

     n = n * 2
  end do

999 format (es10.2 "|" es12.4 ":"  2(3es10.2, "|"), es10.2 )
998 format ("experimental orders of convergence :" 3f8.4 )
end program Newton_Cotttes

subroutine Midpoint(a, b, f1)
  real, intent(in) :: a, b
  real, intent(inout) :: f1
  
  f1 = F((a+b)/2.) * (b-a)
  
end subroutine Midpoint

subroutine Trapezoid(a, b, f1)
  real, intent(in) :: a, b
  real, intent(inout) :: f1
  
  f1 = (F(a) + F(b))/2. * (b-a)
  
end subroutine Trapezoid

subroutine Simpson(a, b, f1)
  real, intent(in) :: a, b
  real, intent(inout) :: f1
  
  f1 = (F(a) + 4. *F((a+b)/2.) +  F(b))/6. * (b-a)
  
end subroutine Simpson


subroutine Exact(a, b, f1)
  real, intent(in) :: a, b
  real, intent(inout) :: f1
  
  f1 = F_primitiv(b) - F_primitiv(a)

end subroutine Exact

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function F(x)
  real :: x

  !F = 1.
  !F = x
  !F = x**2
  !F = x**3
  !F = x**4
  F = exp(x)
  !F = sqrt(x)

end function F

function F_primitiv(x)
  real :: x

  !F_primitiv = x
  !F_primitiv = x**2/2.
  !F_primitiv = x**3/3.
  !F_primitiv = x**4/4.
  !F_primitiv = x**5/5.
  F_primitiv = exp(x) 
  !F_primitiv = x**(3./2) * 2./ 3

end function F_primitiv
